#include	"compiler.h"
#include	<ddraw.h>
#include	"resource.h"
#include	"np2.h"
#include	"winloc.h"
#include	"soundmng.h"
#include	"sysmng.h"
#include	"menu.h"
#include	"ini.h"
#include	"np2class.h"
#include	"dd2.h"
#include	"keydisp.h"
#include	"keydisps.h"
#include	"keydispr.h"


extern WINLOCEX np2_winlocexallwin(HWND base);


static const char np2kdcaption[] = "Key Display";
static const char np2kdclass[] = "NP2-KeyDispWin";
static const char str_kdclose[] = "&Close";

enum {
	IDM_KDCLOSE				= 3000
};

enum {
	KDWIN_MODENONE = 0,
	KDWIN_MODEFM,
	KDWIN_MODEMIDI
};

typedef struct {
	HWND		hwnd;
	WINLOCEX	wlex;
	BYTE		mode;
} KDWIN;

		KEYDISP		keydisp;
static	KDWIN		kdwin;

static	DD2_T	dd_kb;

static const POINT kdpos = {1, 1};

static void _keydisp_out(HWND hWnd, BYTE cnt) {

	RECT	rect;
	RECT	draw;

	GetClientRect(hWnd, &rect);
	draw.left = 0;
	draw.top = 0;
	draw.right = min(KEYDISP_WIDTH, rect.right - rect.left - 1);
	draw.bottom = min(KEYDISP_HEIGHT, rect.bottom - rect.top - 1);
	if ((draw.right <= 0) || (draw.bottom <= 0)) {
		return;
	}
	if (!dd2_bsurflock(&dd_kb)) {
		keydisps_put(&dd_kb.vram, cnt);
		dd2_bsurfunlock(&dd_kb);
		dd2_blt(hWnd, &dd_kb, &kdpos, &draw);
	}
}

static int getdrawheight(BYTE mode) {

	int		height;

	switch(mode) {
		case KDWIN_MODEFM:
			height = kd_posmax;
			break;

		case KDWIN_MODEMIDI:
			height = 16;
			break;

		default:
			height = 0;
			break;
	}
	if (height > KEYDISP_CHMAX) {
		height = KEYDISP_CHMAX;
	}
	height *= KEYDISP_KEYCY;
	return(height);
}

static void setkdwinsize(BYTE mode) {

	int			height;
	WINLOCEX	wlex;

	wlex = np2_winlocexallwin(hWndMain);
	winlocex_setholdwnd(wlex, kdwin.hwnd);
	height = getdrawheight(mode);
	winloc_setclientsize(kdwin.hwnd, KEYDISP_WIDTH + 1, height + 1);
	winlocex_move(wlex);
	winlocex_destroy(wlex);
}

static void setkeydispmode(BYTE mode) {

	HMENU	hmenu;

	if (kdwin.mode != mode) {
		kdwin.mode = mode;
		keydisps_clear();
		if (mode == KDWIN_MODEFM) {
			keydispr_delayreinit();
			keydispr_fminit();
			keydispr_psginit();
		}
		setkdwinsize(mode);
		hmenu = np2class_gethmenu(kdwin.hwnd);
		CheckMenuItem(hmenu, IDM_KDISPFM,
						((mode == KDWIN_MODEFM)?MF_CHECKED:MF_UNCHECKED));
		CheckMenuItem(hmenu, IDM_KDISPMIDI,
						((mode == KDWIN_MODEMIDI)?MF_CHECKED:MF_UNCHECKED));
	}
	else {
		keydisps_alloff();
	}
}

static void kdtypechange(HWND hWnd, BYTE type) {

	WINLOCEX	wlex;

	wlex = np2_winlocexallwin(hWndMain);
	winlocex_setholdwnd(wlex, hWnd);
	np2class_windowtype(hWnd, (type & 1) << 1);
	winlocex_move(wlex);
	winlocex_destroy(wlex);
}

static void kdpaintmsg(HWND hWnd) {

	HDC			hdc;
	PAINTSTRUCT	ps;
	RECT		rect;
	HBRUSH		hbrush;

	keydisps_reload();
	hdc = BeginPaint(hWnd, &ps);
	GetClientRect(hWnd, &rect);
	hbrush = (HBRUSH)SelectObject(hdc, GetStockObject(BLACK_BRUSH));
	PatBlt(hdc, 0, 0, rect.right, 1, PATCOPY);
	PatBlt(hdc, 0, 0, 1, rect.bottom, PATCOPY);
	SelectObject(hdc, hbrush);
	_keydisp_out(hWnd, 0);
	EndPaint(hWnd, &ps);
}

static void openpopup(HWND hWnd, LPARAM lp) {

	HMENU	hMenu;
	POINT	pt;

	hMenu = CreatePopupMenu();
	menu_addmenubar(hMenu, np2class_gethmenu(hWnd));
	AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(hMenu, MF_STRING, IDM_KDCLOSE, str_kdclose);
	pt.x = LOWORD(lp);
	pt.y = HIWORD(lp);
	ClientToScreen(hWnd, &pt);
	TrackPopupMenu(hMenu, TPM_LEFTALIGN, pt.x, pt.y, 0, hWnd, NULL);
	DestroyMenu(hMenu);
}

static LRESULT CALLBACK kdproc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp) {

	switch(msg) {
		case WM_CREATE:
			np2class_wmcreate(hWnd);
			np2class_windowtype(hWnd, (keydisp.type & 1) << 1);
			break;

		case WM_COMMAND:
			switch(LOWORD(wp)) {
				case IDM_KDISPFM:
					keydisp.mode = KEYDISP_FM;
					sysmng_update(SYS_UPDATEOSCFG);
					setkeydispmode(KDWIN_MODEFM);
					break;

				case IDM_KDISPMIDI:
					keydisp.mode = KEYDISP_MIDI;
					sysmng_update(SYS_UPDATEOSCFG);
					setkeydispmode(KDWIN_MODEMIDI);
					break;

				case IDM_KDCLOSE:
					return(SendMessage(hWnd, WM_CLOSE, 0, 0));
			}
			break;

		case WM_PAINT:
			kdpaintmsg(hWnd);
			break;

		case WM_ACTIVATE:
			if (LOWORD(wp) != WA_INACTIVE) {
				keydisps_reload();
				_keydisp_out(hWnd, 0);
			}
			break;

		case WM_LBUTTONDOWN:
			if (keydisp.type & 1) {
				return(SendMessage(hWnd, WM_NCLBUTTONDOWN, HTCAPTION, 0L));
			}
			break;

		case WM_RBUTTONDOWN:
			openpopup(hWnd, lp);
			break;

		case WM_LBUTTONDBLCLK:
			keydisp.type ^= 1;
			kdtypechange(hWnd, keydisp.type);
			sysmng_update(SYS_UPDATEOSCFG);
			break;

		case WM_KEYDOWN:
		case WM_KEYUP:
			SendMessage(hWndMain, msg, wp, lp);
			break;

		case WM_ENTERMENULOOP:
			soundmng_disable(SNDPROC_KEYDISP);
			break;

		case WM_EXITMENULOOP:
			soundmng_enable(SNDPROC_KEYDISP);
			break;

		case WM_ENTERSIZEMOVE:
			soundmng_disable(SNDPROC_KEYDISP);
			winlocex_destroy(kdwin.wlex);
			kdwin.wlex = np2_winlocexallwin(hWnd);
			break;

		case WM_MOVING:
			winlocex_moving(kdwin.wlex, (RECT *)lp);
			break;

		case WM_EXITSIZEMOVE:
			winlocex_destroy(kdwin.wlex);
			kdwin.wlex = NULL;
			soundmng_enable(SNDPROC_KEYDISP);
			break;

		case WM_MOVE:
			if (!(GetWindowLong(hWnd, GWL_STYLE) &
									(WS_MAXIMIZE | WS_MINIMIZE))) {
				RECT rc;
				GetWindowRect(hWnd, &rc);
				keydisp.posx = rc.left;
				keydisp.posy = rc.top;
				sysmng_update(SYS_UPDATEOSCFG);
			}
			break;

		case WM_CLOSE:
			sysmenu_setkeydisp(0);
			sysmng_update(SYS_UPDATEOSCFG);
			DestroyWindow(hWnd);
			break;

		case WM_DESTROY:
			np2class_wmdestroy(hWnd);
			dd2_release(&dd_kb);
			kdwin.hwnd = NULL;
			kdwin.mode = KDWIN_MODENONE;
			break;

		default:
			return(DefWindowProc(hWnd, msg, wp, lp));
	}
	return(0L);
}


// ----

BOOL keydisp_initialize(HINSTANCE hPreInst) {

	WNDCLASS	wc;

	ZeroMemory(&dd_kb, sizeof(DD2_T));
	if (!hPreInst) {
		ZeroMemory(&wc, sizeof(wc));
		wc.style = CS_BYTEALIGNCLIENT | CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		wc.lpfnWndProc = kdproc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = NP2GWL_SIZE;
		wc.hInstance = hInst;
		wc.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON2));
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground = (HBRUSH)GetStockObject(NULL_BRUSH);
		wc.lpszMenuName = MAKEINTRESOURCE(IDR_KEYDISP);
		wc.lpszClassName = np2kdclass;
		if (!RegisterClass(&wc)) {
			return(FAILURE);
		}
	}
	return(SUCCESS);
}

void keydisp_create(void) {

	HWND	hwnd;
	BYTE	mode;

	if (kdwin.hwnd != NULL) {
		return;
	}
	ZeroMemory(&kdwin, sizeof(kdwin));
	hwnd = CreateWindow(np2kdclass, np2kdcaption,
						WS_OVERLAPPED | WS_SYSMENU | WS_CAPTION |
						WS_MINIMIZEBOX,
						keydisp.posx, keydisp.posy,
						CW_USEDEFAULT, CW_USEDEFAULT,
						NULL, NULL, hInst, NULL);
	kdwin.hwnd = hwnd;
	if (hwnd == NULL) {
		goto kdcre_err1;
	}
	switch(keydisp.mode) {
		case KEYDISP_FM:
		default:
			mode = KDWIN_MODEFM;
			break;

		case KEYDISP_MIDI:
			mode = KDWIN_MODEMIDI;
			break;
	}
	setkeydispmode(mode);
	ShowWindow(hwnd, SW_SHOWNOACTIVATE);
	UpdateWindow(hwnd);
	if (dd2_create(hwnd, &dd_kb, KEYDISP_WIDTH, KEYDISP_HEIGHT) != SUCCESS) {
		goto kdcre_err2;
	}
	keydisps_init();
	keydisps_palinit(&dd_kb);
	keydisp_draw(0);
	SetForegroundWindow(hWndMain);
	return;

kdcre_err2:
	DestroyWindow(hwnd);

kdcre_err1:
	sysmenu_setkeydisp(0);
	sysmng_update(SYS_UPDATEOSCFG);
}

void keydisp_destroy(void) {

	if (kdwin.hwnd != NULL) {
		DestroyWindow(kdwin.hwnd);
	}
}

HWND keydisp_gethwnd(void) {

	return(kdwin.hwnd);
}

void keydisp_draw(BYTE cnt) {

	if (kdwin.hwnd) {
		if (!cnt) {
			cnt = 1;
		}
		if (kdwin.mode == KDWIN_MODEFM) {
			keydispr_fmsync();
			keydispr_psgsync();
			keydispr_delay(cnt);
		}
		_keydisp_out(kdwin.hwnd, cnt);
	}
}

void keydisp_setfmboard(BYTE board) {

	keydispr_setfmboard(board);
	if (kdwin.mode == KDWIN_MODEFM) {
		setkdwinsize(KDWIN_MODEFM);
	}
}

void keydisp_fmkeyon(BYTE ch, BYTE value) {

	if (kdwin.mode == KDWIN_MODEFM) {
		keydispr_fmkeyon(ch, value);
	}
}

void keydisp_psgmix(void *psg) {

	if (kdwin.mode == KDWIN_MODEFM) {
		keydispr_psgmix(psg);
	}
}

void keydisp_psgvol(void *psg, BYTE ch) {

	if (kdwin.mode == KDWIN_MODEFM) {
		keydispr_psgvol(psg, ch);
	}
}

void keydisp_midi(const BYTE *cmd) {

	if (kdwin.mode == KDWIN_MODEMIDI) {
		keydispr_midi(cmd);
	}
}


// ----

static const char ini_title[] = "NP2 keydisp";

static const INITBL iniitem[] = {
	{"WindposX", INITYPE_SINT32,	&keydisp.posx,			0},
	{"WindposY", INITYPE_SINT32,	&keydisp.posy,			0},
	{"keydmode", INITYPE_UINT8,		&keydisp.mode,			0},
	{"windtype", INITYPE_BOOL,		&keydisp.type,			0}};

void keydisp_readini(void) {

	char	path[MAX_PATH];

	ZeroMemory(&keydisp, sizeof(keydisp));
	keydisp.posx = CW_USEDEFAULT;
	keydisp.posy = CW_USEDEFAULT;
	initgetfile(path, sizeof(path));
	ini_read(path, ini_title, iniitem, sizeof(iniitem)/sizeof(INITBL));
}

void keydisp_writeini(void) {

	char	path[MAX_PATH];

	initgetfile(path, sizeof(path));
	ini_write(path, ini_title, iniitem, sizeof(iniitem)/sizeof(INITBL));
}

